package jpa.test.annotation.inheritance;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;

import jpa.test.client.ClientJPABase;
import jpa.test.annotation.inheritance.BaseEntityCalss;
import jpa.test.annotation.inheritance.SubEntityClass1;
import jpa.test.annotation.inheritance.SubEntityClass2;

public class Client extends ClientJPABase
{
    public Client() {}

    public void testMethod001()
    {
        EntityManagerFactory emf = null;
        EntityManager em = null;

        testStart("Inheritance-Test001");
        try
        {
            setPersistenceUnitName("TestPersistenceUnit");
            em = getEntityManager();

            SubEntityClass1 entity1 = em.find(SubEntityClass1.class, "ID001-001");
            SubEntityClass2 entity2 = em.find(SubEntityClass2.class, "ID001-002");

            assertEquals("BaseDetail-001-001",      entity1.superClassData);
            assertEquals("SubClass1Detail-001-001", entity1.subClassData);

            assertEquals("BaseDetail-001-002",      entity2.superClassData);
            assertEquals("SubClass2Detail-001-002", entity2.subClassData);
        }
        catch(Throwable thw)
        {
            failed("unexpected error occurred",thw);
            thw.printStackTrace();
        }
        finally
        {
            testEnd();
            cleanup();
        }
    }

    public void testMethod002()
    {
        EntityManagerFactory emf = null;
        EntityManager em = null;

        testStart("Inheritance-Test002");
        try
        {
            setPersistenceUnitName("TestPersistenceUnit");
            em = getEntityManager();

            SubEntityClass1 entity1 = em.find(SubEntityClass1.class, "ID002-003");
            SubEntityClass2 entity2 = em.find(SubEntityClass2.class, "ID002-004");
            assertNull(entity1);
            assertNull(entity2);

            entity1 = em.find(SubEntityClass1.class, "ID002-000");
            entity2 = em.find(SubEntityClass2.class, "ID002-000");
            assertNull(entity1);
            assertNull(entity2);

            BaseEntityCalss entity0 = em.find(BaseEntityCalss.class, "ID002-000");
            assertEquals("BaseDetail-002-000", entity0.superClassData);
        }
        catch(Throwable thw)
        {
            failed("unexpected error occurred",thw);
            thw.printStackTrace();
        }
        finally
        {
            testEnd();
            cleanup();
        }
    }


    public void testMethod003()
    {
        EntityManagerFactory emf = null;
        EntityManager em = null;

        testStart("Inheritance-Test003");
        try
        {
            setPersistenceUnitName("TestPersistenceUnit");

            em = getEntityManager();

            BaseEntityCalss entity0 = new BaseEntityCalss();
            SubEntityClass1 entity1 = new SubEntityClass1();
            SubEntityClass2 entity2 = new SubEntityClass2();

            entity0.id = "ID003-000";
            entity0.superClassData = "InsertBaseClassData-003-000";
            entity1.id = "ID003-001";
            entity1.superClassData = "InsertBaseClassData-003-001";
            entity1.subClassData = "InsertSubClassData-003-001";
            entity2.id = "ID003-002";
            entity2.superClassData = "InsertBaseClassData-003-002";
            entity2.subClassData = "InsertSubClassData-003-002";

            em.persist(entity0);
            em.persist(entity1);
            em.persist(entity2);

            transactionBegin();
            em.flush();
            transactionCommit();

            // Keep using the same EM, it returns the data that PC holds.
            // To confirm the state of the table on DB, once delete PC.
            // Keep using the same PC, it avoid the problem that causes internal wrong.
            cleanup();

            em = getEntityManager();

            SubEntityClass1 entity3 = em.find(SubEntityClass1.class, "ID003-001");
            SubEntityClass2 entity4 = em.find(SubEntityClass2.class, "ID003-002");
            BaseEntityCalss entity5 = em.find(BaseEntityCalss.class, "ID003-000");

            assertEquals("InsertBaseClassData-003-001", entity3.superClassData);
            assertEquals("InsertSubClassData-003-001", entity3.subClassData);
            assertEquals("InsertBaseClassData-003-002", entity4.superClassData);
            assertEquals("InsertSubClassData-003-002", entity4.subClassData);
            assertEquals("InsertBaseClassData-003-000", entity5.superClassData);

            BaseEntityCalss entity_1 = em.find(BaseEntityCalss.class, "ID003-001");
            BaseEntityCalss entity_2 = em.find(BaseEntityCalss.class, "ID003-002");
            assertEquals("InsertBaseClassData-003-001", entity_1.superClassData);
            assertEquals("InsertBaseClassData-003-002", entity_2.superClassData);

            try
            {
                SubEntityClass1 entity_3 = em.find(SubEntityClass1.class, "ID003-000");
                assertNull(entity_3);
            }
            catch(Throwable thw)
            {
                failed("known bug. The expected value is null, but keeping using the same EM will cause ClassCastException",thw);
            }

            try
            {
                SubEntityClass2 entity_4 = em.find(SubEntityClass2.class, "ID003-000");
                assertNull(entity_4);
            }
            catch(Throwable thw)
            {
                failed("known bug. The expected value is null, but keeping using the same EM will cause ClassCastException",thw);
            }

            try
            {
                SubEntityClass2 entity_3 = em.find(SubEntityClass2.class, "ID003-001");
                assertNull(entity_3);
            }
            catch(Throwable thw)
            {
                failed("known bug. The expected value is null, but keeping using the same EM will cause ClassCastException",thw);
            }

            try
            {
                SubEntityClass1 entity_4 = em.find(SubEntityClass1.class, "ID003-002");
                assertNull(entity_4);

            }
            catch(Throwable thw)
            {
                failed("known bug. The expected value is null, but keeping using the same EM will cause an exception.",thw);
            }
        }
        catch(Throwable thw)
        {
            failed("unexpected error occurred",thw);
            thw.printStackTrace();
        }
        finally
        {
            testEnd();
            cleanup();
        }
    }


    public static void main(String[] args)
    {
        Client client = new Client();
        client.testMethod001();
        client.testMethod002();
        client.testMethod003();
    }

}
